/*
 * GaussianNoise.cpp
 *
 *  Created on: 2022/06/02
 *      Author: T.Anazawa
 *      Outline: Add gaussian noise for wheels.
 */
#include "../include/NoiseGenerator.h"
#include <fstream>
#include <iostream>

NoiseGenerator::NoiseGenerator()
{
	np = new NoiseParam;
	nh_.param("/choreonoid/noise/detection_rate", np->detect_rate, 1.0);
	nh_.param("/choreonoid/noise/mean", np->mean, 0.0);
	nh_.param("/choreonoid/noise/stddev", np->stddev, 0.0);

	initializeGaussianNoise();
}

NoiseGenerator::~NoiseGenerator()
{
	if(np){
		delete np;
	}
}

void NoiseGenerator::initializeGaussianNoise()
{
	if(np->stddev > 0.0){
		rotation_error_distribution.param(
				std::normal_distribution<>::param_type(np->mean, np->stddev));
	}

	random_num.seed(0);
	detection_probability.reset();
	rotation_error_distribution.reset();
}

//double NoiseGenerator::addGaussianNoise()
std::pair<double, double> NoiseGenerator::addGaussianNoise()
{
	double rotation_error = 0.0;
	double detect_prob = 1.0;

	if(np->stddev > 0.0){
		rotation_error = rotation_error_distribution(random_num);
	}

	if(np->detect_rate < 1.0){
		detect_prob = detection_probability(random_num);
		if(detect_prob > np->detect_rate){
			rotation_error = 0.0;
		}
	}

//	return rotation_error;
	return std::make_pair(rotation_error, detect_prob);
}
